//
// Logic for the addon's options (preferences) management
//
// Author: Jonas Tingeborn
//

var openinchrome = window.openinchrome || {};
openinchrome.Opts = (function(){
  "use strict";

  var PATH_KEY     = "chrome_path",
      TOOLBAR_KEY  = "show_toolbar",
      CONTEXT_KEY  = "show_context",
      DEBUG_KEY    = "debug",
      CLOSE_SINGLE_KEY = "close_on_migrate_single",
      CLOSE_ALL_KEY    = "close_on_migrate_all",
      IGNORE_PINNED_KEY= "ignore_pinned_tabs",
      OPEN_LINKS_KEY   = "openlinks_button",
      OS           = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime).OS,
      prefsMonitor = null,
      prefsService = Components.classes["@mozilla.org/preferences-service;1"]
                    .getService(Components.interfaces.nsIPrefService),
      prefs        = prefsService.getBranch("extensions.openinchrome.");

  ////
  // OS Detection
  function isWindows(){ return OS == "WINNT";  }
  function isLinux()  { return OS == "Linux";  }
  function isOSX()    { return OS == "Darwin"; }

  ////
  // Windows Registry
  function registryReadString(root,key,value) {
    var str, reg;
    try {
      reg = Components.classes["@mozilla.org/windows-registry-key;1"].createInstance(Components.interfaces.nsIWindowsRegKey);
      switch (root) {
        case "HKLM": root = reg.ROOT_KEY_LOCAL_MACHINE; break;
        case "HKCU": root = reg.ROOT_KEY_CURRENT_USER; break;
        default: return;
      }      
      reg.open(root, key, reg.ACCESS_READ);
      str = reg.readStringValue(value);
      reg.close();
      return str;
    } catch(e) {
      return null;
    }
  }

  // If Chrome is installed for "all users" or running as admin, it's retrieved
  // from HKLM. For individual user installs (non-admin), the path can be found
  // in the HKCU branch.
  function getChromePathFromRegistry(){
    var path = registryReadString("HKLM", "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\chrome.exe");
    if(!path) {
      path = registryReadString("HKCU", "SOFTWARE\\Classes\\ChromeHTML\\shell\\open\\command").
             split(/"\s+--\s+"/)[0];
    }
    path = '"' + path.replace(/"/g,'') + '"';     // Ensure path is quoted, so that paths with spaces are properly handled
    return path;
  }

  ////
  // File
  function showFilePicker(){
    var nsIFilePicker = Components.interfaces.nsIFilePicker,
        rv, fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
    fp.init(window, "Locate Google Chrome executable", nsIFilePicker.modeOpen);
    fp.appendFilters(nsIFilePicker.filterAll | nsIFilePicker.filterText);
    rv = fp.show();
    if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace) {
      return fp.file.path;
    }
  }

  function isExecutable(path){
    if(!path) return;
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(path);
    return file.exists() && file.isExecutable();
  }

  function setPath(path){
    if(path && isExecutable(path)) {
      prefs.setCharPref(PATH_KEY,path);
    }
  }

  ////
  // Chrome executable path configuration
  function isConfiguredPath(path){
    return path && path.length > 0;
  }

  ////
  // Retrieves the Chrome browser path.
  // The location for the path is first queried in the user prefs, for when the
  // user has explicitly saved it there. If not previously saved by the user,
  // it will be fetched from the Windows Registry (if on Windows and found in
  // the registry and not explicitly asked to skip this attempt via the
  // optional skipAutoConfigure argument). Then if on Mac, it will be assumed
  // to have been installed the common way (dragged into the apps folder).
  // When all options / attempts fail to find a valid path to Chrome, then
  // undefined is returned.
  function getPath(skipAutoConfigure){
    var path = prefs.getCharPref(PATH_KEY);
    if( isConfiguredPath(path) ) {
      return path;                                         // User has explicitly configured path, so use that.
    } else if(!skipAutoConfigure) {
      if(isWindows()) return getChromePathFromRegistry();  // Try to fetch the path from registry (if on Windows, we may be lucky and find it there)
      if(isOSX()) return '/usr/bin/open -a "Google Chrome"';
    }
  }

  // Splits command string into array on white spaces, except when the arguments
  // are quoted, then each quoted string is regarded as a single argument. The
  // quoted characters are culled, as they are no longer necessary after the
  // tokenization.
  function splitQuoted(path){
    var q="";
    return (path || "").split("").map(function(c){
      if(c==q) { q = ""; return; }                         // End of quoted string
      if(q == "" && (c=="'" || c=='"')) { q = c; return; } // Start of quoted string
      if(q && /\s/.test(c)) { return "\0"; }               // Encode whitespace temporarily in quoted string
      return c;
    }).join("").split(" ").map(function(s){
      return s.replace(/\0/g," ");                         // Restore any temporarily encoded white spaces
    });
  }

  // Convenience function for properly formatting the args to exec
  function getPathAsCommandArray() {
    return splitQuoted(getPath());
  }

  ////
  // UI feedback
  function updatePathColor(){
    var el = document.getElementById('chrome-path'), path;
    if(el) {                                               // Only run when in the context of the the options box
      path = el.value.split(/\s+/)[0];
      path = path.replace(/["']/g,'');                     // Need to remove any quotes, as the browser API won't accept.
      el.className = isExecutable(path) ? "valid" : "";
    }
  }

  // Update the path field in the options pane to show the user the default
  // chrome path used by the extension, for when the user hasn't explicitly
  // configured the path earlier.
  // This is only a transient value (not stored in prefs) and will be
  // discarded unless the user choses to modify the value and apply the change.
  // The function only has an effect on Windows and OSX, where the path is
  // fetched from the registry if present or has a static launch string (OSX).
  // On other OSes it's a noop.
  function updatePathFieldTransiently(){
    if( !isConfiguredPath( getPath(true) ) ) {                       // Skip if the user has explicitly configured the path before.
      var pathField = document.getElementById('chrome-path');
      pathField.value = getPath() || "";                             // Set the field value to what was discovered, if anything, to help the user understand the current config.
    }
  }

  function openSettingsWindow(){
    var modal   = !prefsService.getBranch("browser.preferences.")    // Fix for issue on the OSX wrt modality.
                  .getBoolPref("instantApply"),                      // "Settings windows should never be opened modally when browser.preferences.instantApply is true" ~ Review comment
        dlgOpts = "close=no,chrome,titlebar,toolbar,centerscreen,dialog=yes" + (modal ? ",modal=yes" : "");


    openDialog("chrome://openinchrome/content/options.xul", "Preferences", dlgOpts);
  }

  function ensureConfigured(){
    if(!isConfiguredPath( getPath() )) {            // Other OSes or if chrome auto-discovery fails on windows.
      openSettingsWindow();
    }
    return isConfiguredPath( getPath() );
  }

  /////////////////////////////////////////////////////////////////////////////
  // Preferences Monitoring
  /////////////////////////////////////////////////////////////////////////////
  function startPrefsMonitoring(onChangeCallback){
    prefsMonitor = {
      observe: function(subject, topic, key){
        if(topic != "nsPref:changed") return;
        if(key == TOOLBAR_KEY || 
           key == CONTEXT_KEY || 
           key == DEBUG_KEY) onChangeCallback();
      }
    };
    prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    prefs.addObserver("", prefsMonitor, false);
  }

  function stopPrefsMonitoring() {
    prefs.removeObserver("", prefsMonitor);
  }

  ////
  // Public interface
  return {
    onLoad: function() {
      if(!window.document.getElementById("openinchrome-preferences")) return;  // Only auto-initialize if loaded by the options window.
      updatePathFieldTransiently();
      updatePathColor();
    },
    onPathModification: function(e){
      updatePathColor();
    },
    pickChromeExecutableCommand: function(e){
      setPath(showFilePicker());
    },
    isToolbarEnabled: function(){
      return prefs.getBoolPref(TOOLBAR_KEY);
    },
    isContextMenuEnabled: function(){
      return prefs.getBoolPref(CONTEXT_KEY);
    },
    isDebugEnabled: function(){
      return prefs.getBoolPref(DEBUG_KEY);
    },
    isCloseSingleEnabled: function(){
      return prefs.getBoolPref(CLOSE_SINGLE_KEY);
    },
    isCloseAllEnabled: function(){
      return prefs.getBoolPref(CLOSE_ALL_KEY);
    },
    isIgnorePinnedEnabled: function(){
      return prefs.getBoolPref(IGNORE_PINNED_KEY);
    },
    isMigrateLinksEnabled: function(){
      return prefs.getCharPref(OPEN_LINKS_KEY) != "disabled";
    },
    getClickConfigForCommand: function(cmdName){                               // Currently supports names "opencurrenttab", "openalltabs" and "openlinks"
      return {
        "button" : prefs.getCharPref(cmdName + "_button"),                     // May also have the value "disabled" for the openlinks command name aside from the generic three "left","middle","right"
        "ctrl"   : prefs.getBoolPref(cmdName + "_modifier_ctrl"),
        "alt"    : prefs.getBoolPref(cmdName + "_modifier_alt"),
        "shift"  : prefs.getBoolPref(cmdName + "_modifier_shift")
      };
    },
    ensureConfigured: ensureConfigured,
    getPath: getPath,
    getPathAsCommandArray: getPathAsCommandArray,
    startPrefsMonitoring: startPrefsMonitoring,
    stopPrefsMonitoring:  stopPrefsMonitoring
  };
})();

window.addEventListener("load", openinchrome.Opts.onLoad, false);

